//----------------------------------------------------------------
//
// OpenDCC
//
// Copyright (c) 2006-8 Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-----------------------------------------------------------------
//
// file:      pc_send.c
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// history:   2008-04-04 V0.8 used for sniffer
//
//-----------------------------------------------------------------
//
// purpose:   dcc sniffer
// content:   routines for printf replacement
//            see also:
//            http://www.roboternetz.de/wissen/index.php/UART_mit_avr-gcc
//
//-----------------------------------------------------------------

#include <stdlib.h>
#include <stdbool.h>
#include <inttypes.h>
#include <ctype.h>
#include <avr/pgmspace.h>        // put var to program memory
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <string.h>




#include "config.h"               // general structures and definitions
#include "hardware.h"
#include "rs232.h"

//
//-----------------------------------------------------------------


//---------------------------------------------------------------------------------
// Senderoutinen zum PC

void pc_send(unsigned char out)                     // send single char
  {
    while (!tx_fifo_ready()) ;                      // busy waiting!!!
                                                    // vorlufig kein Timeout
    tx_fifo_write(out);                             // send character
  }


void pc_send_string_P(const char *progmem_string)   // send string in PROGMEM
  {
    char c;

    /* we need to read each byte from program memory */
    while ( (c = pgm_read_byte(progmem_string++)) )
      {
        pc_send(c);
      }
  }



void pc_send_answer_P(const char *progmem_string)   // send string in PROGMEM + cr ]
  {
    char c;

    /* we need to read each byte from program memory */
    while ( (c = pgm_read_byte(progmem_string++)) )
      {
        pc_send(c);
      }
    pc_send(0x0D);
    pc_send(0x0A);
  }


void pc_send_num(unsigned int num)                  // send an int + leading blank
  {
    char buffer[6];         // max. 5 digits
    unsigned char i=0;

    itoa(num, buffer, 10);         // 10 byte long

    pc_send(' ');
    while (buffer[i])
      {
        pc_send(buffer[i++]);
      }
  }

void pc_send_int(unsigned int num)                  // send an int
  {
    char buffer[6];         // max. 5 digits
    unsigned char i=0;

    itoa(num, buffer, 10);         // 10 byte long

    while (buffer[i])
      {
        pc_send(buffer[i++]);
      }
  }


const char hexdig[] PROGMEM = "0123456789ABCDEF";

void pc_send_hex(unsigned char value)                  // send a char as hex, leading 0
  {
    unsigned char i=0;

    i = value >> 4;
    i &= 0x0f;
    pc_send(pgm_read_byte(&hexdig[i]));

    i = value & 0x0f;
    pc_send(pgm_read_byte(&hexdig[i]));
  }

void pc_send_bin(unsigned char value)                  // send a char as bin, leading 0
  {
    unsigned char i=0;

    for (i=0; i<8; i++)
      {
        if (value & 0x80)    pc_send('1');
        else                 pc_send('0');

        value = value << 1;
      }
  }
